#!/bin/csh -f
#
# This script is an example of how to set up processing for the 
# the IMAPP Level 2 MODIS NIR water vapor Processing.
#
#  This script:
#  1) Extracts binary radiance/reflectance, geolocation and 
#     metadata information from MODIS DAAC or direct broadcast
#     HDF files.
#  2) Runs the IMAPP MODIS NIR water vapor software.
#
# Originally written (for SST) by James E. Davies  January 2004
# Jim.Davies@ssec.wisc.edu
# Adapted for NIR water vapor by Peter Albert September 2004
# peter.albert@gmx.dw
#
# Search for !!! for places where directoy locations are set and will
#  need to be changed for your on-site processing. 
#-------------------------------------------------------------------------------
# SETUP
#-------------------------------------------------------------------------------

# Check arguments
if ($#argv != 4) then
  echo "Usage: modis_wvnir.csh SAT FIL1KM FILGEO OUTDIR"
  echo "where"
  echo "SAT is the MODIS platform name (Terra or Aqua)"
  echo "FIL1KM is the MODIS L1B 1000 meter resolution radiance flat file"
  echo "FILGEO is the MODIS L1B 1000 meter resolution geolocation flat file"
  echo "OUTDIR is the directory where the products will be stored"
  exit(-1)
endif

# Extract file names
set SAT = $argv[1]
set FIL1KM = $argv[2]
set FILGEO = $argv[3]
set OUTDIR = $argv[4]

# Set root of output file name (e.g. 't1.02001.1815')
set ROOT = $FIL1KM:r
set ROOT = $ROOT:r
set ROOT = $ROOT:t

#----------------------
# RUN THE WVNIR ALGORITHM
#----------------------

# Print start message for processing log
echo
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo "Started IMAPP MODIS NIR water vapor processing at "`date`

# Extract flat files
#echo "Started flatfile extraction at "`date`

#!!! Set directory for flatfile extraction data and src !!!
set SRCDIR = ../../flatfile/src
set DATA_DIR = ../../data

$SRCDIR/modis_extract_1km.exe $DATA_DIR/$FIL1KM $SAT ${ROOT}.1000m.img ${ROOT}.1000m.hdr
$SRCDIR/modis_extract_geo.exe $DATA_DIR/$FILGEO $SAT ${ROOT}.geo.img ${ROOT}.geo.hdr
echo "Finished flatfile extraction at "`date`
 
# Set WVNIR output file names
set OUTWVNIRFIL   = $ROOT.wvnir.img
set OUTWVNIRHDR   = $ROOT.wvnir.hdr

# Create a configuration file from the template
#!!! Set wvnir src directory !!!
set SRCDIR = ../src
set TEMPLATE = $SRCDIR/wvnir.cfg
/bin/sed \
  -e "s/FIL1KM_IMG/${ROOT}.1000m.img/g" \
  -e "s/FIL1KM_HDR/${ROOT}.1000m.hdr/g" \
  -e "s/FILGEO_IMG/${ROOT}.geo.img/g" \
  -e "s/FILGEO_HDR/${ROOT}.geo.hdr/g" \
  -e "s/FILWVNIR_IMG/${OUTWVNIRFIL}/g" \
  -e "s/FILWVNIR_HDR/${OUTWVNIRHDR}/g" \
  $TEMPLATE > wvnir.cfg

# Run the WV retrieval executable
$SRCDIR/wvnir.exe wvnir.cfg

# Rename the configuration file
/bin/mv wvnir.cfg $ROOT.wvnir.cfg

# Move the output files to the product directory
/bin/mv $OUTWVNIRFIL $OUTWVNIRHDR $ROOT.wvnir.cfg $OUTDIR

#-------------------------------------------------------------------------------
# CLEANUP AND EXIT
#-------------------------------------------------------------------------------

# Print end message for processing log
echo
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo "Finished IMAPP MODIS NIR water vapor processing at "`date`

# Exit gracefully
exit(0)
